"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const LemonStore_1 = require("./LemonStore");
const is_plain_object_1 = __importDefault(require("is-plain-object"));
const defaultStoreConfig = {
    name: 'DefaultStore',
    debug: false
};
const devToolsConect = (typeof window !== 'undefined' &&
    window.__REDUX_DEVTOOLS_EXTENSION__ &&
    window.__REDUX_DEVTOOLS_EXTENSION__.connect) || (() => { });
class LemonState {
    constructor(initial, actions = {}, config = defaultStoreConfig) {
        this.config = config;
        this.actions = {};
        this.isRemoved = false;
        this.devTools = undefined;
        this.dispatch = (actionFunction, payload) => {
            const result = actionFunction({
                getState: this.store.getState.bind(this.store),
                setState: this.store.setState.bind(this.store),
                dispatch: this.dispatch.bind(this.store)
            }, payload);
            if (is_plain_object_1.default(result)) {
                this.store.setState(result);
            }
            if (this.devTools) {
                this.devTools.send(actionFunction.name, this.store.getState());
            }
            return result;
        };
        this.getState = () => {
            return this.store.getState();
        };
        this.setState = (diff) => {
            this.store.setState(diff);
        };
        this.store = new LemonStore_1.LemonStore(initial, config.name);
        this.config = Object.assign(Object.assign({}, defaultStoreConfig), config);
        Object.keys(actions).forEach((actionKey) => {
            const action = actions[actionKey];
            if (typeof action !== 'function') {
                throw new Error(`Action ${actionKey} must be a function (in ${this.config.name})`);
            }
            Object.defineProperty(this.actions, actionKey, {
                enumerable: true,
                configurable: true,
                get: () => {
                    return (payload) => this.dispatch(action, payload);
                },
                set: () => {
                    throw new Error(`Can't modify actions (in ${this.config.name})`);
                }
            });
        });
        if (config.debug) {
            this.devTools = devToolsConect({
                name: config.name,
                features: {
                    jump: true
                }
            });
            if (this.devTools) {
                this.devTools.init(this.store.getState());
                this.devTools.subscribe((message) => {
                    if (message.type === 'DISPATCH' && message.state) {
                        this.store.setState(JSON.parse(message.state));
                    }
                });
            }
        }
    }
    subscribe(subscriber) {
        return this.store.subscribe(subscriber);
    }
    smartSubscribe(subscriber) {
        return this.store.smartSubscribe(subscriber);
    }
    remove() {
        const throwAccessError = () => {
            throw new Error(`'${this.config.name}' is removed!`);
        };
        this.store.remove();
        this.isRemoved = true;
        this.actions = {};
        this.dispatch = throwAccessError;
    }
}
exports.LemonState = LemonState;
