"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class SharedData {
    constructor() {
        this.removedValues = new Set();
        this.stores = new Map();
        this.subscribersMap = new Map();
        this.proxyLinkUpdaters = new Map();
        this.computeChainProp = new Set();
        this.changedValues = new Set();
        this.needUpdateValues = new Set();
        this.valueMap = new Map();
        this.activeComputingValueId = undefined;
        this.resolvedValues = undefined;
        this.nullLastValueSymbol = Symbol('Null last value');
    }
    clearWork() {
        this.changedValues.clear();
        this.needUpdateValues.clear();
        this.activeComputingValueId = undefined;
        this.resolvedValues = undefined;
        this.computeChainProp.clear();
    }
    ;
    getStoreById(id) {
        return this.stores.get(id);
    }
    ;
    throwError(ErrorConstructor, message) {
        this.clearWork();
        throw new ErrorConstructor(message);
    }
    ;
    getValueInfo(valueId) {
        const valueInfo = this.valueMap.get(valueId);
        if (this.removedValues.has(valueId)) {
            const removedStore = this.getStoreById(valueInfo.store);
            this.throwError(ReferenceError, `There is no access to '${valueInfo.name}' (in ${removedStore.name})`);
        }
        return valueInfo;
    }
    ;
    getValueInfoByStateProp(stateValues, propName) {
        const valueId = stateValues.get(propName);
        return this.getValueInfo(valueId);
    }
    ;
    registerChangedValue(valueId) {
        this.changedValues.add(valueId);
    }
    registerStore(storeId, store, getSubscribers, getUpdateProxyLink) {
        this.stores.set(storeId, store);
        this.subscribersMap.set(storeId, getSubscribers);
        this.proxyLinkUpdaters.set(storeId, getUpdateProxyLink);
    }
    removeStoreValues(values) {
        values.forEach(valueId => {
            const valueInfo = this.getValueInfo(valueId);
            valueInfo.depends.forEach(dependId => {
                const dependInfo = this.getValueInfo(dependId);
                dependInfo.invoking.delete(valueId);
            });
            valueInfo.invoking.forEach(invokeId => {
                const invokeInfo = this.getValueInfo(invokeId);
                invokeInfo.depends.delete(valueId);
            });
            valueInfo.cachedValue = this.nullLastValueSymbol;
            valueInfo.value = this.nullLastValueSymbol;
            valueInfo.depends = new Set();
            valueInfo.invoking = new Set();
            this.removedValues.add(valueId);
        });
    }
    ;
    removeStoreSubscribers(storeId) {
        this.subscribersMap.set(storeId, () => []);
    }
    removeStoreProxyLinkUpdater(storeId) {
        this.proxyLinkUpdaters.set(storeId, () => { });
    }
    canSetState() {
        return !this.resolvedValues && !this.activeComputingValueId;
    }
}
exports.SharedData = SharedData;
;
