"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const propertyListener_1 = require("./propertyListener");
class SmartSubscriber {
    constructor(state) {
        this.smartSubscribersTriggers = new Map();
        this.smartSubscribers = new Map();
        this.noIndexedSubscribers = new Set();
        state.subscribe(this.mainSubscriber.bind(this));
    }
    smartUnsubscribe(subscriber) {
        this.noIndexedSubscribers.delete(subscriber);
        this.smartSubscribers.get(subscriber).forEach(key => {
            this.smartSubscribersTriggers.get(key).delete(subscriber);
            if (!this.smartSubscribersTriggers.get(key).size) {
                this.smartSubscribersTriggers.delete(key);
            }
        });
        this.smartSubscribers.delete(subscriber);
    }
    indexSubscriber(subscriber, state, changedProps) {
        const { getProxy, getUsedkeys, stop } = propertyListener_1.propertyListener(state);
        subscriber(getProxy(), changedProps);
        stop();
        return getUsedkeys();
    }
    mainSubscriber(newState, changedProps) {
        const needToCall = new Set();
        changedProps.forEach(prop => {
            if (this.smartSubscribersTriggers.has(prop)) {
                this.smartSubscribersTriggers.get(prop).forEach(needToCall.add, needToCall);
            }
        });
        needToCall.forEach(subscriber => subscriber(newState, changedProps));
        this.noIndexedSubscribers.forEach(subscriber => {
            const keysToIndex = this.indexSubscriber(subscriber, newState, changedProps);
            keysToIndex.forEach(key => {
                if (!this.smartSubscribersTriggers.get(key)) {
                    this.smartSubscribersTriggers.set(key, new Set());
                }
                this.smartSubscribersTriggers.get(key).add(subscriber);
            });
            this.noIndexedSubscribers.delete(subscriber);
        });
    }
    smartSubscribe(subscriber) {
        this.noIndexedSubscribers.add(subscriber);
        return () => this.smartUnsubscribe(subscriber);
    }
    remove() {
        this.smartSubscribers.clear();
        this.smartSubscribersTriggers.clear();
        this.noIndexedSubscribers.clear();
    }
}
exports.SmartSubscriber = SmartSubscriber;
