export declare type SimpleFunc = (...args: any) => any;
export declare type ComputeFunc<S, T> = (state: S) => T;
export declare type LemonStoreInit<T> = {
    [K in keyof T]: T[K] extends SimpleFunc ? ComputeFunc<T, ReturnType<T[K]>> : T[K];
};
export declare type State<T> = {
    [K in keyof T]: LemonStoreInit<T>[K] extends SimpleFunc ? ReturnType<LemonStoreInit<T>[K]> : T[K];
};
export declare type ValueInfo = {
    id: symbol;
    type: ValueType;
    store: symbol;
    name: string;
    invoking: Set<symbol>;
    depends: Set<symbol>;
    cachedValue: any;
    value: any;
};
export declare enum ValueType {
    STATIC = "static",
    COMPUTED = "computed"
}
export interface Subscriber<T> {
    (newState: State<T>, changedProps: Set<keyof T>): void;
}
export interface Unsubscribe {
    (): void;
}
export interface StoreChange<T> {
    getState: () => State<T>;
    setState: (newState: Partial<T>) => void;
    dispatch: Dispatch<T>;
}
export interface Action<T> {
    (storeChange: StoreChange<T>, payload?: any): Partial<T> | any;
}
export declare type BoundActions<A> = {
    [P in keyof A]: (payload?: any) => any;
};
export declare type Actions<T, A> = {
    [P in keyof A]: Action<T>;
};
export interface StoreConfig {
    name?: string;
    debug?: boolean;
}
export interface Dispatch<T> {
    (actionFunction: Action<T>, payload?: any): void;
}
