"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("./types");
exports.getValueCreator = (sharedData, valueGetter) => {
    const createProtectedSetter = (storeName) => {
        const chachedSetters = new Map();
        if (!chachedSetters.has(storeName)) {
            chachedSetters.set(storeName, (_) => {
                sharedData.throwError(ReferenceError, `You can't modify state directly (in ${storeName})`);
            });
        }
        return chachedSetters.get(storeName);
    };
    const activateAccessors = (storeInfo, valueId, propName) => {
        const propertyConfig = {
            enumerable: true,
            configurable: true,
            get: valueGetter(valueId),
            set: createProtectedSetter(storeInfo.name),
        };
        Object.defineProperty(storeInfo.proxy, propName, propertyConfig);
    };
    const createNewValue = (storeInfo, value, propName, notComputed = false) => {
        const isComputed = typeof value === 'function' && !notComputed;
        const propId = Symbol(`${propName} (in ${storeInfo.name})`);
        sharedData.valueMap.set(propId, {
            id: propId,
            store: storeInfo.storeId,
            name: propName,
            depends: new Set(),
            invoking: new Set(),
            cachedValue: isComputed ? sharedData.nullLastValueSymbol : value,
            type: isComputed ? types_1.ValueType.COMPUTED : types_1.ValueType.STATIC,
            value
        });
        activateAccessors(storeInfo, propId, propName);
        return propId;
    };
    return {
        createNewValue
    };
};
