"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const propertyListener_1 = require("../Store/propertyListener");
const hasIntersection = (set1, set2) => {
    const smallestSet = set1.size < set2.size ? set1 : set2;
    const biggestSet = set1 === smallestSet ? set2 : set1;
    let result = false;
    smallestSet.forEach((key) => {
        if (biggestSet.has(key)) {
            result = true;
        }
    });
    return result;
};
exports.createHook = (store) => {
    const useStore = () => {
        const listenObject = propertyListener_1.propertyListener(store.getState());
        const [state, setState] = react_1.useState(listenObject.getProxy());
        const firstInitUnsubscriber = react_1.useRef(null);
        if (!firstInitUnsubscriber.current) {
            firstInitUnsubscriber.current = store.subscribe((newState, changedProps) => {
                listenObject.stop();
                if (firstInitUnsubscriber.current) {
                    firstInitUnsubscriber.current();
                }
                if (hasIntersection(changedProps, listenObject.getUsedkeys())) {
                    setState(newState);
                }
            });
        }
        react_1.useEffect(() => {
            listenObject.stop();
            if (firstInitUnsubscriber.current) {
                firstInitUnsubscriber.current();
            }
            return store.subscribe((newState, changedProps) => {
                if (hasIntersection(changedProps, listenObject.getUsedkeys())) {
                    setState(newState);
                }
            });
        }, []);
        return Object.assign(state, store.actions);
    };
    return { useStore };
};
